// Calvo Model with Uncertainty Shocks

// Endogenous Varialbes
var Z sigma_Z C N w R pi Y BB CC p_star Delta mc Y_f Y_g r
    lambda_1 lambda_2 lambda_3 lambda_4 lambda_5 lambda_6 lambda_7 lambda_8 lambda_9;

// Exogenous Variables
varexo e_Z e_sigma_Z;

// Parameters
parameters beta gamma eta chi epsilon tau theta xi omega Z_ss C_ss N_ss w_ss R_ss pi_ss Y_ss BB_ss CC_ss mc_ss p_star_ss Delta_ss rho_Z sigma_Z_ss rho_sigma_Z sigma_sigma_Z
    lambda_1_ss lambda_2_ss lambda_3_ss lambda_4_ss lambda_5_ss lambda_6_ss lambda_7_ss lambda_8_ss lambda_9_ss;

xi = 0;     // trend infation if xi=0 // past inflation if xi=1
omega = 1;  // indexation
gamma = 2;
eta = 1;
epsilon = 11;
tau = 1/epsilon;
theta = 0.75;
Z_ss = 1;
beta = 0.99;
pi_ss = 1;
R_ss = pi_ss/beta;
N_ss = 1/3;
p_star_ss = ((1-theta*(((pi_ss^omega)/pi_ss)^(1-epsilon)))/(1-theta))^(1/(1-epsilon));
w_ss = Z_ss*p_star_ss*((epsilon-1)/epsilon)/(1-tau)*((1-theta*beta*(pi_ss/(pi_ss^omega))^epsilon)/(1-theta*beta*(pi_ss/(pi_ss^omega))^(epsilon-1)));
Delta_ss = ((1-theta)*p_star_ss^(-epsilon))/(1-theta*(pi_ss/(pi_ss^omega))^epsilon);
Y_ss = Z_ss*N_ss/Delta_ss;
C_ss = Y_ss;
chi = w_ss*N_ss^(-eta)*C_ss^(-gamma);
BB_ss = (C_ss^(-gamma)*Y_ss)/(1-theta*beta*((pi_ss^omega)/pi_ss)^(1-epsilon));  
CC_ss = (C_ss^(-gamma)*Y_ss*w_ss/Z_ss)/(1-theta*beta*((pi_ss^omega)/pi_ss)^(-epsilon));
mc_ss = w_ss/Z_ss;
lambda_2_ss = (-C_ss^(-gamma) + chi*N_ss^eta)/(chi*N_ss^eta*gamma*C_ss^(gamma-1) + chi*eta*N_ss^(eta-1)*C_ss^gamma);
lambda_1_ss = lambda_2_ss;
lambda_3_ss = 0;
lambda_6_ss=  lambda_1_ss*(Z_ss)/(-C_ss^(-gamma)*Y_ss);
lambda_4_ss = lambda_6_ss*(theta-1)/(-epsilon/(epsilon-1))/(1-tau);
lambda_5_ss   = lambda_4_ss/(theta-1);
lambda_7_ss = (-chi*N_ss^eta + lambda_2_ss*(chi*eta*N_ss^(eta-1)*C_ss^gamma))/Z_ss;
lambda_8_ss = -lambda_7_ss*Y_ss/(1-beta*theta);
lambda_9_ss = lambda_5_ss*(C_ss^(-gamma)) + lambda_6_ss*(mc_ss*C_ss^(-gamma)) - lambda_7_ss*Delta_ss;
rho_Z = 0.95;           % Persistency of first moment shock
sigma_Z_ss = 0.01;   % Mean value of volatility of shock
rho_sigma_Z = 0.76;    % Persistence of uncertainty shock
sigma_sigma_Z = 0.392;  % Standard deviation of uncertainty shock

model;
 
// 1. Intratemporal Optimality Condition
    chi*(N^eta)*(C^gamma) = w;

// 2. Intertemporal Optimality Condition
    C^(-gamma) = beta*(C(+1)^(-gamma))*(R)/pi(+1);

// 3. New Keynesian Phillips Curve
    p_star = (epsilon/(epsilon-1))*(1-tau)*CC/BB;

// 4.
    BB = (C^(-gamma))*Y + theta*beta*((pi(+1))^(epsilon-1))*BB(+1);

// 5.
    CC = (C^(-gamma))*Y*mc + theta*beta*((pi(+1))^(epsilon))*CC(+1);

// 6. Aggregate Price Index
    1 = (1-theta)*p_star^(1-epsilon) + theta*(pi)^(epsilon-1);

// 7. Production Function
    Delta*Y = Z*N;

// 8. Price Dispersion
    Delta = (1-theta)*p_star^(-epsilon) + theta*((pi)^epsilon)*Delta(-1);

// 9. Good Markets Clearing
    Y = C ;

// 10. Markup
    mc = w/Z;

// 11. fexible Price Level of Output
    chi*(Y_f/Z)^(eta) = Y_f^(-gamma)*(epsilon-1)/epsilon*Z/(1-tau);

// 12. Output Gap
    Y_g = Y/Y_f;

// 12.
    r = (R)/pi(+1);


// 11. N
    -chi*N^eta + lambda_2*(chi*eta*N^(eta-1)*C^gamma) + lambda_7*(-Z) = 0;

// 12. R
    lambda_3 = 0;

// 13. mc
    lambda_1*(-Z) + lambda_6*(-Y*C^(-gamma)) = 0;

// 14. w
    lambda_1 + lambda_2*(-1) = 0;

// 15. Y
    lambda_5*(-C^(-gamma)) + lambda_6*(-mc*C^(-gamma)) + lambda_7*(Delta) + lambda_9 = 0;

// 16. C
    C^(-gamma) + lambda_2*(chi*N^eta*gamma*C^(gamma-1)) + lambda_3*(-gamma*C^(-gamma-1)*pi(+1)) + lambda_5*(-Y*(-gamma)*C^(-gamma-1))
    + lambda_6*(-Y*mc*(-gamma)*C^(-gamma-1)) + lambda_9*(-1) + lambda_3(-1)*(gamma*C^(-gamma-1)*R(-1)) = 0;

// 17. BB
    lambda_4*(((1-theta*pi^(epsilon-1))/(1-theta))^(1/(1-epsilon))) + lambda_5 + lambda_5(-1)*(-theta*pi^(epsilon-1)) = 0;

// 18. CC
    lambda_4*(-epsilon/(epsilon-1))*(1-tau) + lambda_6 + lambda_6(-1)*(-theta*pi^epsilon) = 0;

// 19. Delta
    lambda_7*Y + lambda_8 + beta*lambda_8(+1)*(-theta*pi(+1)^epsilon) = 0;

// 20. pi
    lambda_4*(1/(1-epsilon)*((1-theta*pi^(epsilon-1))/(1-theta))^(1/(1-epsilon) - 1)*(-theta/(1-theta)*(epsilon-1)*pi^(epsilon-2))*BB)
    + lambda_8*( -(1-theta)*epsilon/(epsilon-1)*((1 - theta*pi^(epsilon-1))/(1-theta))^(epsilon/(epsilon-1) - 1)*(-theta/(1-theta)*(epsilon-1)*pi^(epsilon-2))   - theta*epsilon*pi^(epsilon-1)*Delta(-1))
    + beta^(-1)*lambda_3(-1)*C(-1)^(-gamma) + lambda_5(-1)*(-theta*(epsilon-1)*pi^(epsilon-2)*BB)
    + lambda_6(-1)*(-theta*epsilon*pi^(epsilon-1)*CC) = 0;

// 21. Stochastic process for technology level shock
    log(Z) = rho_Z*log(Z(-1)) + sigma_Z*e_Z;

// 22. Stochastic process for technology second moment shock
    log(sigma_Z) = (1-rho_sigma_Z)*log(sigma_Z_ss) + rho_sigma_Z*log(sigma_Z(-1)) + sigma_sigma_Z*e_sigma_Z;

end;

initval;
Z = Z_ss;
sigma_Z = sigma_Z_ss;
C = C_ss;
N = N_ss;
w = w_ss;
R = R_ss;
pi = pi_ss;
Y = Y_ss;
BB = BB_ss;
CC = CC_ss;
p_star = p_star_ss;
Delta = Delta_ss;
mc = mc_ss;
Y_f = Y_ss;
Y_g = 1;
r = (R_ss)/pi_ss;
lambda_1 = lambda_1_ss;
lambda_2 = lambda_2_ss;
lambda_3 = lambda_3_ss;
lambda_4 = lambda_4_ss;
lambda_5 = lambda_5_ss;
lambda_6 = lambda_6_ss;
lambda_7 = lambda_7_ss;
lambda_8 = lambda_8_ss;
lambda_9 = lambda_9_ss;
end;

resid(1);
steady;
check;

shocks;
var e_Z = 1;
var e_sigma_Z = 1;
end;


//stoch_simul(order=3,periods=0,irf=0,noprint,nograph,nomoments,nofunctions,nocorr,pruning);
stoch_simul(order=3,pruning,nograph);



irflength           = 21;
irfburninlength     = 2000;

oo_.stochastic_steady_state = sss(oo_.dr, irfburninlength, options_.order);

for ivariable = 1:M_.endo_nbr
    assignin('base',[deblank(M_.endo_names(ivariable,:)) '_sss'],...
                     oo_.stochastic_steady_state(ivariable,:)');                     
end

for ishock = 1:M_.exo_nbr    

    variables_irfsss  =  irfsss(oo_.dr,M_.Sigma_e(:,ishock),irflength,irfburninlength,options_.order)';

    for ivariable = 1:M_.endo_nbr
        assignin('base',[deblank(M_.endo_names(ivariable,:)) '_' deblank(M_.exo_names(ishock,:))],...
                          variables_irfsss(:,ivariable));                         
    end

end

t = 0:irflength-1;

figure(1)
subplot(3,2,1)
hold on
plot(t,100*Y_e_sigma_Z./Y_sss,'-','Color','r','LineWidth',3)
hold on
plot(zeros(irflength,1),'k--','HandleVisibility','off'); 
hold on
xlabel('Quarter','FontSize',12);xlim([t(1) t(end)]);set(gca,'XTick',[0:4:irflength],'FontSize',12);
ylabel('Percent','FontSize',12)
title('Output','FontSize',16);

figure(1)
subplot(3,2,2)
hold on
plot(t,400*pi_e_sigma_Z,'-','Color','r','LineWidth',3)
hold on
plot(zeros(irflength,1),'k--','HandleVisibility','off');
hold on
xlabel('Quarter','FontSize',12);xlim([t(1) t(end)]);set(gca,'XTick',[0:4:irflength],'FontSize',12);
ylabel('Percentage Points','FontSize',12)
title('Inflation','FontSize',16);

figure(1)
subplot(3,2,3)
hold on
plot(t,400*R_e_sigma_Z,'-','Color','r','LineWidth',3)
hold on
plot(zeros(irflength,1),'k--','HandleVisibility','off');
hold on
xlabel('Quarter','FontSize',12);xlim([t(1) t(end)]);set(gca,'XTick',[0:4:irflength],'FontSize',12);
ylabel('Percentage Points','FontSize',12)
title('Nominal Interest Rate','FontSize',16);

subplot(3,2,4)
hold on
plot(t,400*r_e_sigma_Z,'-','Color','r','LineWidth',3)
hold on
plot(zeros(irflength,1),'k--','HandleVisibility','off'); 
hold on
xlabel('Quarter','FontSize',12);xlim([t(1) t(end)]);set(gca,'XTick',[0:4:irflength],'FontSize',12);
ylabel('Percent','FontSize',12)
title('Real Interest Rate','FontSize',16);

figure(1)
subplot(3,2,5)
hold on
plot(t,100*Y_g_e_sigma_Z./Y_g_sss,'-','Color','r','LineWidth',3)
hold on
plot(zeros(irflength,1),'k--','HandleVisibility','off'); 
hold on
xlabel('Quarter','FontSize',12);xlim([t(1) t(end)]);set(gca,'XTick',[0:4:irflength],'FontSize',12);
ylabel('Percent','FontSize',12)
title('Output Gap','FontSize',16);

figure(1)
subplot(3,2,6)
hold on
plot(t,100*sigma_Z_e_sigma_Z./sigma_Z_sss,'-','Color','r','LineWidth',3)
hold on
plot(zeros(irflength,1),'k--','HandleVisibility','off'); 
hold on
xlabel('Quarter','FontSize',12);xlim([t(1) t(end)]);set(gca,'XTick',[0:4:irflength],'FontSize',12);
ylabel('Percent','FontSize',12)
title('Uncertainty Shock','FontSize',16);
