// Rotemberg Model with Uncertainty Shocks

// Endogenous Varialbes
var Z sigma_Z C N w R pi Y mc Y_f Y_g r
    lambda_1 lambda_2 lambda_3 lambda_4 lambda_5 lambda_6;

// Exogenous Variables
varexo e_Z e_sigma_Z;

// Parameters
parameters beta gamma eta chi epsilon tau theta xi omega phi Z_ss C_ss N_ss w_ss R_ss pi_ss Y_ss AC_ss mc_ss rho_Z sigma_Z_ss rho_sigma_Z sigma_sigma_Z
    lambda_1_ss lambda_2_ss lambda_3_ss lambda_4_ss lambda_5_ss lambda_6_ss;

xi = 0;     // trend infation if xi=0 // past inflation if xi=1
omega = 1;  // indexation
gamma = 2;
eta = 1;
epsilon = 11;
tau = 1/epsilon;
Z_ss = 1;
beta = 0.99;
pi_ss = 1;
R_ss = pi_ss/beta;
N_ss = 1/3;
theta = 0.75;
phi = (theta*(epsilon-1))/((1-theta)*(1-beta*theta));
Y_ss = Z_ss*N_ss;
w_ss = Z_ss*((epsilon-1)/epsilon/(1-tau) + ((1-beta)*phi*(pi_ss^(1-omega)-1)*pi_ss^(1-omega))/epsilon);
AC_ss = (phi/2)*((pi_ss^(1-omega) - 1)^2)*Y_ss;
C_ss = Y_ss  - AC_ss;
mc_ss = w_ss/Z_ss;
chi = w_ss*N_ss^(-eta)*C_ss^(-gamma);
lambda_2_ss = (-C_ss^(-gamma) + chi*N_ss^eta)/(chi*N_ss^eta*gamma*C_ss^(gamma-1) + chi*eta*N_ss^(eta-1)*C_ss^gamma);
lambda_1_ss =lambda_2_ss;
lambda_3_ss =0;
lambda_4_ss =lambda_1_ss*(Z_ss)/(-epsilon*(1-tau)*C_ss^(-gamma)*Y_ss);
lambda_5_ss   = -chi*N_ss^eta + lambda_2_ss*chi*eta*N_ss^(eta-1)*C_ss^gamma;
lambda_6_ss=  - lambda_5_ss;

rho_Z = 0.95;           % Persistency of first moment shock
sigma_Z_ss = 0.01;   % Mean value of volatility of shock
rho_sigma_Z = 0.76;    % Persistence of uncertainty shock
sigma_sigma_Z = 0.392;  % Standard deviation of uncertainty shock

model;

// 1. Intratemporal Optimality Condition
    chi*(N^eta)*(C^gamma) = w;

// 2. Intertemporal Optimality Condition
    C^(-gamma) = beta*(C(+1)^(-gamma))*R/pi(+1);

// 3. New Keynesian Phillips Curve
    phi*(pi - 1)*(pi) = phi*beta*((C(+1)/C)^(-gamma))*(pi(+1) - 1)*(pi(+1))*(Y(+1)/Y) + 1 - epsilon + epsilon*(1-tau)*mc;

// 4. Production Function
    Y = Z*N;

// 5. Good Markets Clearing
    Y = C + (phi/2)*((pi - 1)^2)*Y;

// 6. Marginal Costs
    mc = w/Z;

// 11. fexible Price Level of Output
    chi*(Y_f/Z)^(eta) = Y_f^(-gamma)*(epsilon-1)/epsilon*Z/(1-tau);

// 12. Output Gap
    Y_g = Y/Y_f;

// 12.
    r = (R)/pi(+1);


// 7. N
    - chi*N^eta + lambda_2*chi*eta*N^(eta-1)*C^gamma + lambda_5*(-Z) = 0;

// 8. R
    lambda_3*(-beta*C(+1)^(-gamma)) = 0;

// 9. mc
    lambda_1*(-Z) + lambda_4*(-epsilon*(1-tau)*C^(-gamma)*Y) = 0;

// 10. w
    lambda_1 + lambda_2*(-1) = 0;

// 11. C
    C^(-gamma) + lambda_2*(chi*N^eta*gamma*C^(gamma-1)) + lambda_3*(-gamma*C^(-gamma-1)*pi(+1)) 
    + lambda_4*(phi*(pi-1)*pi*(-gamma)*C^(-gamma-1)*Y - (1-epsilon)*(-gamma)*C^(-gamma-1)*Y - epsilon*(1-tau)*mc*(-gamma)*C^(-gamma-1)*Y)
    + lambda_6*(-1) + lambda_3(-1)*(-(-gamma)*C^(-gamma-1)*R(-1)) + lambda_4(-1)*(-phi*(-gamma)*C^(-gamma-1)*(pi-1)*pi*Y) = 0;

// 12. Y
    lambda_4*(phi*(pi-1)*pi*C^(-gamma) - (1-epsilon)*C^(-gamma) - epsilon*(1-tau)*mc*C^(-gamma))
    +lambda_5 + lambda_6*(1 - phi/2*(pi-1)^2) + lambda_4(-1)*(-phi*C^(-gamma)*(pi-1)*pi) = 0;

// 13. pi
    lambda_4*(phi*pi*C^(-gamma)*Y + phi*(pi-1)*C^(-gamma)*Y) + lambda_6*(-phi*(pi-1)*Y)
    +lambda_4(-1)*(-phi*C^(-gamma)*pi*Y - phi*C^(-gamma)*(pi-1)*Y) + beta^(-1)*lambda_3(-1)*C(-1)^(-gamma) = 0;

// 14. Stochastic process for technology level shock
    log(Z) = (1-rho_Z)*log(Z_ss) + rho_Z*log(Z(-1)) + sigma_Z*e_Z;

// 15. Stochastic process for technology second moment shock
    log(sigma_Z) = (1-rho_sigma_Z)*log(sigma_Z_ss) + rho_sigma_Z*log(sigma_Z(-1)) + sigma_sigma_Z*e_sigma_Z;

end;

initval;
Z = Z_ss;
sigma_Z = sigma_Z_ss;
C = C_ss;
N = N_ss;
w = w_ss;
R = R_ss;
pi = pi_ss;
Y = Y_ss;
mc = mc_ss;
Y_f = Y_ss;
Y_g = 1;
r = (R_ss)/pi_ss;
lambda_1 = lambda_1_ss;
lambda_2 = lambda_2_ss;
lambda_3 = lambda_3_ss;
lambda_4 = lambda_4_ss;
lambda_5 = lambda_5_ss;
lambda_6 = lambda_6_ss;
end;

resid(1);
steady;
check;

shocks;
var e_Z = 1;
var e_sigma_Z = 1;
end;


//stoch_simul(order=3,periods=0,irf=0,noprint,nograph,nomoments,nofunctions,nocorr,pruning,k_order_solver);
stoch_simul(order=3,pruning,nograph);



irflength           = 21;
irfburninlength     = 2000;

oo_.stochastic_steady_state = sss(oo_.dr, irfburninlength, options_.order);

for ivariable = 1:M_.endo_nbr
    assignin('base',[deblank(M_.endo_names(ivariable,:)) '_sss'],...
                     oo_.stochastic_steady_state(ivariable,:)');                     
end

for ishock = 1:M_.exo_nbr    

    variables_irfsss  =  irfsss(oo_.dr,M_.Sigma_e(:,ishock),irflength,irfburninlength,options_.order)';

    for ivariable = 1:M_.endo_nbr
        assignin('base',[deblank(M_.endo_names(ivariable,:)) '_' deblank(M_.exo_names(ishock,:))],...
                          variables_irfsss(:,ivariable));                         
    end

end

t = 0:irflength-1;

figure(1)
subplot(3,2,1)
hold on
plot(t,100*Y_e_sigma_Z./Y_sss,'--','Color','b','LineWidth',3)
hold on
plot(zeros(irflength,1),'k--','HandleVisibility','off'); 
hold on
xlabel('Quarter','FontSize',12);xlim([t(1) t(end)]);set(gca,'XTick',[0:4:irflength],'FontSize',12);
ylabel('Percent','FontSize',12)
title('Output','FontSize',16);


figure(1)
subplot(3,2,2)
hold on
plot(t,400*pi_e_sigma_Z,'--','Color','b','LineWidth',3)
hold on
plot(zeros(irflength,1),'k--','HandleVisibility','off');
hold on
xlabel('Quarter','FontSize',12);xlim([t(1) t(end)]);set(gca,'XTick',[0:4:irflength],'FontSize',12);
ylabel('Percentage Points','FontSize',12)
title('Inflation','FontSize',16);

figure(1)
subplot(3,2,3)
hold on
plot(t,400*R_e_sigma_Z,'--','Color','b','LineWidth',3)
hold on
plot(zeros(irflength,1),'k--','HandleVisibility','off');
hold on
xlabel('Quarter','FontSize',12);xlim([t(1) t(end)]);set(gca,'XTick',[0:4:irflength],'FontSize',12);
ylabel('Percentage Points','FontSize',12)
title('Nominal Interest Rate','FontSize',16);

subplot(3,2,4)
hold on
plot(t,400*r_e_sigma_Z,'--','Color','b','LineWidth',3)
hold on
plot(zeros(irflength,1),'k--','HandleVisibility','off'); 
hold on
xlabel('Quarter','FontSize',12);xlim([t(1) t(end)]);set(gca,'XTick',[0:4:irflength],'FontSize',12);
ylabel('Percent','FontSize',12)
title('Real Interest Rate','FontSize',16);

figure(1)
subplot(3,2,5)
hold on
plot(t,100*Y_g_e_sigma_Z./Y_g_sss,'--','Color','b','LineWidth',3)
hold on
plot(zeros(irflength,1),'k--','HandleVisibility','off'); 
hold on
xlabel('Quarter','FontSize',12);xlim([t(1) t(end)]);set(gca,'XTick',[0:4:irflength],'FontSize',12);
ylabel('Percent','FontSize',12)
title('Output Gap','FontSize',16);

figure(1)
subplot(3,2,6)
hold on
plot(t,100*sigma_Z_e_sigma_Z./sigma_Z_sss,'--','Color','b','LineWidth',3)
hold on
plot(zeros(irflength,1),'k--','HandleVisibility','off'); 
hold on
xlabel('Quarter','FontSize',12);xlim([t(1) t(end)]);set(gca,'XTick',[0:4:irflength],'FontSize',12);
ylabel('Percent','FontSize',12)
title('Uncertainty Shock','FontSize',16);
legend('Calvo','Rotemberg','Location','NorthEast')

set(gcf, 'Color', 'w');
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperUnits', 'inches');
set(gcf, 'PaperOrientation', 'portrait');
set(gcf, 'PaperPosition', [-0.35 -0.75 9.5 12.0]);
print('-dpdf',['ramsey_results.pdf'])

