// Rotemberg Model with Uncertainty Shocks

// Endogenous Varialbes
var Z sigma_Z C N w i pi Y AC mc Y_f Y_g r;

// Exogenous Variables
varexo e_Z e_sigma_Z;

// Parameters
parameters beta gamma eta chi epsilon tau theta xi omega phi rho_i rho_pi rho_Y Z_ss C_ss N_ss w_ss i_ss pi_ss Y_ss Y_f_ss AC_ss mc_ss rho_Z sigma_Z_ss rho_sigma_Z sigma_sigma_Z;

xi = 0;     // trend infation if xi=0 // past inflation if xi=1
omega = 1;  // indexation
gamma = 2;
eta = 1;
epsilon = 11;
tau = 1/epsilon;
Z_ss = 1;
beta = 0.99;
pi_ss = 1;
i_ss = pi_ss/beta-1;
N_ss = 1/3;
theta = 0.75;
phi = (theta*(epsilon-1))/((1-theta)*(1-beta*theta));
Y_ss = Z_ss*N_ss;
w_ss = Z_ss*((epsilon-1)/epsilon/(1-tau) + ((1-beta)*phi*(pi_ss^(1-omega)-1)*pi_ss^(1-omega))/epsilon);
AC_ss = 1+(phi/2)*((pi_ss^(1-omega) - 1)^2)*Y_ss;
C_ss = Y_ss ;
mc_ss = w_ss/Z_ss;
chi = w_ss*N_ss^(-eta)*C_ss^(-gamma);
Y_f_ss = Y_ss;
rho_i = 0;
rho_pi = 1.5;
rho_Y = 0;
rho_Z = 0.95;           % Persistency of first moment shock
sigma_Z_ss = 0.01;   % Mean value of volatility of shock
rho_sigma_Z = 0.76;    % Persistence of uncertainty shock
sigma_sigma_Z = 0.392;  % Standard deviation of uncertainty shock

model;
// 1. Intratemporal Optimality Condition
    chi*(N^eta)*(C^gamma) = w;

// 2. Intertemporal Optimality Condition
    C^(-gamma) = beta*(C(+1)^(-gamma))*(1+i)/pi(+1);

// 3. New Keynesian Phillips Curve
    phi*(pi/(pi_ss^(omega*(1-xi))*pi(-1)^(omega*xi)) - 1)*(pi/(pi_ss^(omega*(1-xi))*pi(-1)^(omega*xi))) = phi*beta*((C(+1)/C)^(-gamma))*(pi(+1)/(pi_ss^(omega*(1-xi))*pi^(omega*xi)) - 1)*(pi(+1)/(pi_ss^(omega*(1-xi))*pi^(omega*xi)))*(Y(+1)/Y) + 1 - epsilon + epsilon*(1-tau)*mc;

// 4. Production Function
    Y = Z*N;

// 5. Good Markets Clearing
    Y = C + (phi/2)*((pi/(pi_ss^(omega*(1-xi))*pi(-1)^(omega*xi)) - 1)^2)*Y;

// 6. Adjustment Costs
    AC = 1 + phi/2*(pi - 1)^2*Y;

// 7. Marginal Costs
    mc = w/Z;

// 8. fexible Price Level of Output
    chi*(Y_f/Z)^(eta) = Y_f^(-gamma)*(epsilon-1)/epsilon*Z/(1-tau);

// 9. Output Gap
    Y_g = Y/Y_f;

// 13. real interest rate
    r = (1+i)/pi(+1);

// 6. Taylor Rules
    (1+i)/(1+i_ss) = (((1+i(-1))/(1+i_ss))^rho_i)*(pi^(rho_pi*(1-rho_i)))*(Y_g^(rho_Y*(1-rho_i)));

// 17. Stochastic process for technology level shock
    log(Z) = (1-rho_Z)*log(Z_ss) + rho_Z*log(Z(-1)) + sigma_Z*e_Z;

// 18. Stochastic process for technology second moment shock
    log(sigma_Z) = (1-rho_sigma_Z)*log(sigma_Z_ss) + rho_sigma_Z*log(sigma_Z(-1)) + sigma_sigma_Z*e_sigma_Z;

end;

initval;
Z = Z_ss;
sigma_Z = sigma_Z_ss;
C = C_ss;
N = N_ss;
w = w_ss;
i = i_ss;
pi = pi_ss;
Y = Y_ss;
AC = AC_ss;
mc = mc_ss;
Y_f = Y_f_ss;
Y_g = 1;
r = (1+i_ss)/pi_ss;
end;

resid(1);
steady;
check;

shocks;
var e_Z = 1;
var e_sigma_Z = 1;
end;


//stoch_simul(order=3,periods=0,irf=0,noprint,nograph,nomoments,nofunctions,nocorr,pruning,k_order_solver);
stoch_simul(order=3,pruning,nograph);



irflength           = 21;
irfburninlength     = 2000;

oo_.stochastic_steady_state = sss(oo_.dr, irfburninlength, options_.order);

for ivariable = 1:M_.endo_nbr
    assignin('base',[deblank(M_.endo_names(ivariable,:)) '_sss'],...
                     oo_.stochastic_steady_state(ivariable,:)');                     
end

for ishock = 1:M_.exo_nbr    

    variables_irfsss  =  irfsss(oo_.dr,M_.Sigma_e(:,ishock),irflength,irfburninlength,options_.order)';

    for ivariable = 1:M_.endo_nbr
        assignin('base',[deblank(M_.endo_names(ivariable,:)) '_' deblank(M_.exo_names(ishock,:))],...
                          variables_irfsss(:,ivariable));                         
    end

end

t = 0:irflength-1;

figure(1)
subplot(3,2,1)
hold on
plot(t,100*Y_e_sigma_Z./Y_sss,'--','Color','b','LineWidth',3)
hold on
plot(zeros(irflength,1),'k--','HandleVisibility','off'); 
hold on
xlabel('Quarter','FontSize',12);xlim([t(1) t(end)]);set(gca,'XTick',[0:4:irflength],'FontSize',12);
ylabel('Percent','FontSize',12)
title('Output','FontSize',16);


figure(1)
subplot(3,2,2)
hold on
plot(t,400*pi_e_sigma_Z,'--','Color','b','LineWidth',3)
hold on
plot(zeros(irflength,1),'k--','HandleVisibility','off');
hold on
xlabel('Quarter','FontSize',12);xlim([t(1) t(end)]);set(gca,'XTick',[0:4:irflength],'FontSize',12);
ylabel('Percentage Points','FontSize',12)
title('Inflation','FontSize',16);

figure(1)
subplot(3,2,3)
hold on
plot(t,400*i_e_sigma_Z,'--','Color','b','LineWidth',3)
hold on
plot(zeros(irflength,1),'k--','HandleVisibility','off');
hold on
xlabel('Quarter','FontSize',12);xlim([t(1) t(end)]);set(gca,'XTick',[0:4:irflength],'FontSize',12);
ylabel('Percentage Points','FontSize',12)
title('Nominal Interest Rate','FontSize',16);

subplot(3,2,4)
hold on
plot(t,400*r_e_sigma_Z,'--','Color','b','LineWidth',3)
hold on
plot(zeros(irflength,1),'k--','HandleVisibility','off'); 
hold on
xlabel('Quarter','FontSize',12);xlim([t(1) t(end)]);set(gca,'XTick',[0:4:irflength],'FontSize',12);
ylabel('Percentage Point','FontSize',12)
title('Real Interest Rate','FontSize',16);

figure(1)
subplot(3,2,5)
hold on
plot(t,100*Y_g_e_sigma_Z./Y_g_sss,'--','Color','b','LineWidth',3)
hold on
plot(zeros(irflength,1),'k--','HandleVisibility','off'); 
hold on
xlabel('Quarter','FontSize',12);xlim([t(1) t(end)]);set(gca,'XTick',[0:4:irflength],'FontSize',12);
ylabel('Percent','FontSize',12)
title('Output Gap','FontSize',16);

figure(1)
subplot(3,2,6)
hold on
plot(t,100*sigma_Z_e_sigma_Z./sigma_Z_sss,'--','Color','b','LineWidth',3)
hold on
plot(zeros(irflength,1),'k--','HandleVisibility','off'); 
hold on
xlabel('Quarter','FontSize',12);xlim([t(1) t(end)]);set(gca,'XTick',[0:4:irflength],'FontSize',12);
ylabel('Percent','FontSize',12)
title('Uncertainty Shock','FontSize',16);



